"""StarRail battle chronicle component."""

import asyncio
import typing

from genshin import errors, types, utility
from genshin.client import routes
from genshin.models import zzz as models

from . import base

__all__ = ("ZZZBattleChronicleClient",)


class ZZZBattleChronicleClient(base.BaseBattleChronicleClient):
    """ZZZ battle chronicle component."""

    async def _request_zzz_record(
        self,
        endpoint: str,
        uid: typing.Optional[int] = None,
        *,
        method: str = "GET",
        lang: typing.Optional[str] = None,
        payload: typing.Optional[typing.Mapping[str, typing.Any]] = None,
        cache: bool = False,
        is_nap_ledger: bool = False,
        is_special_payload: bool = False,
    ) -> typing.Mapping[str, typing.Any]:
        """Get an arbitrary ZZZ object."""
        payload = dict(payload or {})
        original_payload = payload.copy()

        uid = uid or await self._get_uid(types.Game.ZZZ)

        if is_nap_ledger or is_special_payload:
            payload = {
                "uid": uid,
                "region": utility.recognize_zzz_server(uid),
                **payload,
            }
        else:
            payload = {
                "role_id": uid,
                "server": utility.recognize_zzz_server(uid),
                **payload,
            }

        data, params = None, None
        if method == "POST":
            data = payload
        else:
            params = payload

        cache_key: typing.Optional[base.ChronicleCacheKey] = None
        if cache:
            cache_key = base.ChronicleCacheKey(
                types.Game.ZZZ,
                endpoint,
                uid,
                lang=lang or self.lang,
                params=tuple(original_payload.values()),
            )

        return await self.request_game_record(
            endpoint,
            lang=lang,
            game=types.Game.ZZZ,
            region=utility.recognize_region(uid, game=types.Game.ZZZ),
            params=params,
            data=data,
            cache=cache_key,
            custom_route=routes.NAP_LEDGER_URL if is_nap_ledger else None,
        )

    @typing.overload
    async def get_zzz_notes(
        self,
        uid: typing.Optional[int] = ...,
        *,
        lang: typing.Optional[str] = ...,
        autoauth: bool = ...,
        return_raw_data: typing.Literal[False] = ...,
    ) -> models.ZZZNotes: ...
    @typing.overload
    async def get_zzz_notes(
        self,
        uid: typing.Optional[int] = ...,
        *,
        lang: typing.Optional[str] = ...,
        autoauth: bool = ...,
        return_raw_data: typing.Literal[True] = ...,
    ) -> typing.Mapping[str, typing.Any]: ...
    async def get_zzz_notes(
        self,
        uid: typing.Optional[int] = None,
        *,
        lang: typing.Optional[str] = None,
        autoauth: bool = True,
        return_raw_data: bool = False,
    ) -> typing.Union[models.ZZZNotes, typing.Mapping[str, typing.Any]]:
        """Get ZZZ sticky notes (real-time notes)."""
        try:
            data = await self._request_zzz_record("note", uid, lang=lang)
        except errors.DataNotPublic as e:
            # error raised only when real-time notes are not enabled
            if uid and (await self._get_uid(types.Game.ZZZ)) != uid:
                raise errors.GenshinException(e.response, "Cannot view real-time notes of other users.") from e
            if not autoauth:
                raise errors.GenshinException(e.response, "Real-time notes are not enabled.") from e

            await self.update_settings(3, True, game=types.Game.ZZZ)
            data = await self._request_zzz_record("note", uid, lang=lang)

        if return_raw_data:
            return data
        return models.ZZZNotes(**data)

    async def get_zzz_diary(
        self,
        uid: typing.Optional[int] = None,
        *,
        month: typing.Optional[str] = None,
        lang: typing.Optional[str] = None,
    ) -> models.ZZZDiary:
        """Get ZZZ inter-knot monthly earning data."""
        data = await self._request_zzz_record(
            "month_info", uid, lang=lang, payload={"month": month or ""}, is_nap_ledger=True
        )
        return models.ZZZDiary(**data)

    async def get_zzz_diary_detail(
        self,
        month: str,
        *,
        type: models.ZZZCurrencyType,
        page: int = 1,
        page_size: int = 20,
        uid: typing.Optional[int] = None,
        lang: typing.Optional[str] = None,
    ) -> models.ZZZDiaryDetail:
        """Get ZZZ inter-knot monthly earning data."""
        if not month:
            raise ValueError("month is required.")

        data = await self._request_zzz_record(
            "month_detail",
            uid,
            lang=lang,
            payload={"month": month, "current_page": page, "type": type.value, "page_size": page_size},
            is_nap_ledger=True,
        )
        return models.ZZZDiaryDetail(**data)

    async def get_zzz_user(
        self,
        uid: typing.Optional[int] = None,
        *,
        lang: typing.Optional[str] = None,
    ) -> models.ZZZUserStats:
        """Get ZZZ user stats."""
        data = await self._request_zzz_record("index", uid, lang=lang)
        return models.ZZZUserStats(**data)

    async def get_zzz_agents(
        self, uid: typing.Optional[int] = None, *, lang: typing.Optional[str] = None
    ) -> typing.Sequence[models.ZZZPartialAgent]:
        """Get all owned ZZZ characters (only brief info)."""
        data = await self._request_zzz_record("avatar/basic", uid, lang=lang)
        return [models.ZZZPartialAgent(**item) for item in data["avatar_list"]]

    async def get_bangboos(
        self, uid: typing.Optional[int] = None, *, lang: typing.Optional[str] = None
    ) -> typing.Sequence[models.ZZZBaseBangboo]:
        """Get all owned ZZZ bangboos."""
        data = await self._request_zzz_record("buddy/info", uid, lang=lang)
        return [models.ZZZBaseBangboo(**item) for item in data["list"]]

    @typing.overload
    async def get_zzz_agent_info(
        self,
        character_id: int,
        *,
        uid: typing.Optional[int] = None,
        lang: typing.Optional[str] = None,
    ) -> models.ZZZFullAgent: ...
    @typing.overload
    async def get_zzz_agent_info(
        self,
        character_id: typing.Sequence[int],
        *,
        uid: typing.Optional[int] = None,
        lang: typing.Optional[str] = None,
    ) -> typing.Sequence[models.ZZZFullAgent]: ...
    async def get_zzz_agent_info(
        self,
        character_id: typing.Union[int, typing.Sequence[int]],
        *,
        uid: typing.Optional[int] = None,
        lang: typing.Optional[str] = None,
    ) -> typing.Union[models.ZZZFullAgent, typing.Sequence[models.ZZZFullAgent]]:
        """Get a ZZZ character's detailed info."""
        if isinstance(character_id, typing.Sequence):
            tasks = [
                self._request_zzz_record("avatar/info", uid, lang=lang, payload={"id_list[]": character_id_})
                for character_id_ in character_id
            ]
            results = await asyncio.gather(*tasks)
            return [models.ZZZFullAgent(**data["avatar_list"][0]) for data in results]

        data = await self._request_zzz_record("avatar/info", uid, lang=lang, payload={"id_list[]": character_id})
        return models.ZZZFullAgent(**data["avatar_list"][0])

    async def get_shiyu_defense(
        self, uid: typing.Optional[int] = None, *, previous: bool = False, lang: typing.Optional[str] = None
    ) -> models.ShiyuDefense:
        """Get ZZZ Shiyu defense stats."""
        payload = {"schedule_type": 2 if previous else 1, "need_all": "true"}
        data = await self._request_zzz_record("challenge", uid, lang=lang, payload=payload)
        return models.ShiyuDefense(**data)

    async def get_deadly_assault(
        self, uid: typing.Optional[int] = None, *, previous: bool = False, lang: typing.Optional[str] = None
    ) -> models.DeadlyAssault:
        """Get ZZZ Shiyu defense stats."""
        payload = {"schedule_type": 2 if previous else 1}
        data = await self._request_zzz_record("mem_detail", uid, lang=lang, payload=payload, is_special_payload=True)
        return models.DeadlyAssault(**data)

    async def get_lost_void_summary(
        self, uid: typing.Optional[int] = None, *, lang: typing.Optional[str] = None
    ) -> models.LostVoidSummary:
        """Get ZZZ Lost Void summary."""
        data = await self._request_zzz_record("abysss2_abstract", uid, lang=lang, is_special_payload=True)
        return models.LostVoidSummary(**data)



import pytest

import genshin


async def test_wish_history(lclient: genshin.Client, authkey: str):
    history = await lclient.wish_history(200, limit=20).flatten()

    assert history[0].banner_type == 200
    assert history[0].banner_name == "Permanent Wish"


async def test_merged_wish_history(lclient: genshin.Client, authkey: str):
    async for wish in lclient.wish_history([200, 302], limit=20):
        assert wish.banner_type in [200, 302]


async def test_banner_types(lclient: genshin.Client, authkey: str):
    banner_types = await lclient.get_banner_names()
    assert sorted(banner_types.keys()) == [100, 200, 301, 302]


async def test_banner_details(lclient: genshin.Client):
    try:
        banners = await lclient.get_banner_details()
    except FileNotFoundError:
        # running in a vm
        pytest.skip("No genshin installation.")

    for details in banners:
        assert details.banner_type in [100, 200, 301, 302, 400, 500]


# async def test_gacha_items(lclient: genshin.Client):
#     items = await lclient.get_genshin_gacha_items()
#     assert items[0].is_character()
#     assert not items[-1].is_character()



import genshin


async def test_record_cards(client: genshin.Client, hoyolab_id: int):
    data = await client.get_record_cards(hoyolab_id)

    assert data

    assert data[0].level >= 40


async def test_honkai_user(honkai_client: genshin.Client, honkai_uid: int):
    data = await honkai_client.get_honkai_user(honkai_uid)

    assert data


async def test_honkai_abyss(honkai_client: genshin.Client, honkai_uid: int):
    data = await honkai_client.get_honkai_abyss(honkai_uid)

    assert data is not None


async def test_elysian_realm(honkai_client: genshin.Client, honkai_uid: int):
    data = await honkai_client.get_elysian_realm(honkai_uid)

    assert data is not None


async def test_memorial_arena(honkai_client: genshin.Client, honkai_uid: int):
    data = await honkai_client.get_memorial_arena(honkai_uid)

    assert data is not None


async def test_full_honkai_user(honkai_client: genshin.Client, honkai_uid: int):
    data = await honkai_client.get_full_honkai_user(honkai_uid)

    assert data



r"""
import typing

import pytest

import genshin


class LiteralCharacter(genshin.models.BaseCharacter): ...


lang = "en-us"  # initiate local scope

@pytest.mark.parametrize(
    ("data", "expected"),
    (
        # complete should give an identity
        (
            {
                "id": 10000002,
                "name": "Kamisato Ayaka",
                "element": "Cryo",
                "rarity": 5,
                "icon": "https://enka.network/ui/UI_AvatarIcon_Ayaka.png",
            },
            LiteralCharacter(
                id=10000002,
                name="Kamisato Ayaka",
                element="Cryo",
                rarity=5,
                icon="https://enka.network/ui/UI_AvatarIcon_Ayaka.png",
            ),
        ),
        # partial should provide the proper name
        (
            {
                "id": 10000003,
                "icon": "https://upload-os-bbs.mihoyo.com/game_record/genshin/character_side_icon/UI_AvatarIcon_Side_Qin.png",
            },
            LiteralCharacter(
                id=10000003,
                name="Jean",
                element="Anemo",
                rarity=5,
                icon="https://enka.network/ui/UI_AvatarIcon_Qin.png",
            ),
        ),
        # traveler element should be kept
        (
            {
                "id": 10000005,
                "name": "Traveler",
                "element": "Light",
                "rarity": 5,
            },
            LiteralCharacter(
                id=10000005,
                name="Traveler",
                element="Light",
                rarity=5,
                icon="https://enka.network/ui/UI_AvatarIcon_PlayerBoy.png",
            ),
        ),
        # messed up icon should be replaced
        (
            {
                "id": 10000041,
                "name": "Mona",
                "element": "Hydro",
                "rarity": 5,
                "icon": "https://uploadstatic-sea.hoyoverse.com/hk4e/e20200928calculate/item_icon_ud09dc/9a8c420feda33f36680839c8b8dc1d83.png",
            },
            LiteralCharacter(
                id=10000041,
                name="Mona",
                element="Hydro",
                rarity=5,
                icon="https://enka.network/ui/UI_AvatarIcon_Mona.png",
            ),
        ),
        # foreign languages should be kept
        (
            {
                "id": 10000046,
                "name": "胡桃",
                "element": "Pyro",
                "rarity": 5,
                "icon": "https://enka.network/ui/UI_AvatarIcon_Hutao.png",
            },
            LiteralCharacter(
                id=10000046,
                name="胡桃",
                element="Pyro",
                rarity=5,
                icon="https://enka.network/ui/UI_AvatarIcon_Hutao.png",
            ),
        ),
    ),
)
def test_genshin_base_character_model(data: typing.Dict[str, typing.Any], expected: genshin.models.BaseCharacter):
    assert genshin.models.BaseCharacter(**data) == expected


# reserialization stuff

all_models: typing.Dict[typing.Type[genshin.models.APIModel], genshin.models.APIModel] = {}


def APIModel___new__(cls: typing.Type[genshin.models.APIModel], *args: typing.Any, **kwargs: typing.Any):
    self = object.__new__(cls)
    all_models[cls] = self
    return self


genshin.models.APIModel.__new__ = APIModel___new__


# def test_model_reserialization():
#     for cls, model in sorted(all_models.items(), key=lambda pair: pair[0].__name__):
#         cls(**model.model_dump())

#         if hasattr(model, "as_dict"):
#             getattr(model, "as_dict")()

#     # dump all parsed models
#     data = ",\n".join(
#         f'"{cls.__name__}": {model.json(indent=4, ensure_ascii=False, models_as_dict=True)}'
#         for cls, model in all_models.items()
#     )
#     data = "{" + data + "}"
#     os.makedirs(".pytest_cache", exist_ok=True)
#     with open(".pytest_cache/hoyo_parsed.json", "w", encoding="utf-8") as file:
#         file.write(data)
"""



import git
import logging

logging.basicConfig(
    level=logging.INFO, format="[%(asctime)s] %(name)s | %(levelname)s - %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
)
logger = logging.getLogger(__name__)

repo = git.Repo()

# Get the last commit message
commit_message = repo.head.commit.message
logger.info(f"Last commit message: {commit_message!r}")

# Is the last commit message a version bump?
if not isinstance(commit_message, str) or "chore(ver)" in commit_message:
    logger.info("Version bump commit found")
    exit()

# Get the current version
current_version = None
with open("pyproject.toml", "r") as f:
    lines = f.readlines()
    for line in lines:
        if line.startswith("version = "):
            current_version = line.split('"')[1]
            break

if current_version is None:
    logger.error("Current version not found in pyproject.toml")
    exit()

logger.info(f"Current version: {current_version!r}")

# Get patch version
major, minor, patch = current_version.split(".")
patch = int(patch) + 1
new_version = f"{major}.{minor}.{patch}"  # e.g. 1.7.5
logger.info(f"New version: {new_version!r}")

# Update the version number in pyproject.toml
with open("pyproject.toml", "w") as f:
    for line in lines:
        if line.startswith("version = "):
            f.write(f'version = "{new_version}"\n')
        else:
            f.write(line)
logger.info("Updated version number in pyproject.toml")



import genshin


async def test_transactions(lclient: genshin.Client, authkey: str):
    log = await lclient.transaction_log("resin", limit=20)
    assert log[0].kind == "resin"
    assert log[0].get_reason_name() != str(log[0].reason_id)


async def test_merged_transactions(lclient: genshin.Client, authkey: str):
    async for trans in lclient.transaction_log(limit=30):
        if trans.kind in ("primogem", "crystal", "resin"):
            assert not hasattr(trans, "name")
        else:
            assert hasattr(trans, "name")



import calendar
import datetime

import pytest

import genshin


async def test_daily_reward(lclient: genshin.Client):
    signed_in, claimed_rewards = await lclient.get_reward_info()

    try:
        reward = await lclient.claim_daily_reward()
    except genshin.DailyGeetestTriggered:
        pytest.skip("Geetest triggered on daily reward.")
    except genshin.AlreadyClaimed:
        assert signed_in
        return
    except genshin.GenshinException as e:
        if e.retcode == -10002:
            pytest.skip("No genshin account.")

        raise
    else:
        assert not signed_in

    rewards = await lclient.get_monthly_rewards()
    assert rewards[claimed_rewards].name == reward.name


async def test_starrail_daily_reward(lclient: genshin.Client):
    signed_in, claimed_rewards = await lclient.get_reward_info(game=genshin.types.Game.STARRAIL)

    try:
        reward = await lclient.claim_daily_reward(game=genshin.types.Game.STARRAIL)
    except genshin.DailyGeetestTriggered:
        pytest.skip("Geetest triggered on daily reward.")
    except genshin.AlreadyClaimed:
        assert signed_in
        return
    except genshin.GenshinException as e:
        if e.retcode == -10002:
            pytest.skip("No star rail account.")

        raise
    else:
        assert not signed_in

    rewards = await lclient.get_monthly_rewards(game=genshin.types.Game.STARRAIL)
    assert rewards[claimed_rewards].name == reward.name


async def test_monthly_rewards(lclient: genshin.Client):
    rewards = await lclient.get_monthly_rewards()
    now = datetime.datetime.now(genshin.constants.CN_TIMEZONE)
    assert len(rewards) == calendar.monthrange(now.year, now.month)[1]


async def test_claimed_rewards(lclient: genshin.Client):
    claimed = await lclient.claimed_rewards(limit=10).flatten()
    assert claimed[0].time <= datetime.datetime.now(genshin.constants.CN_TIMEZONE)



import pytest

import genshin


async def test_game_accounts(lclient: genshin.Client):
    data = await lclient.get_game_accounts()

    assert data


async def test_search(client: genshin.Client, hoyolab_id: int):
    users = await client.search_users("seria_ati")

    for user in users:
        if user.hoyolab_id == hoyolab_id:
            break
    else:
        raise AssertionError("Search did not return the correct users")

    assert user.nickname == "seria_ati"


async def test_hoyolab_user(client: genshin.Client, hoyolab_id: int):
    user = await client.get_hoyolab_user(hoyolab_id)

    assert user.nickname == "seria_ati"


async def test_recommended_users(client: genshin.Client):
    users = await client.get_recommended_users()

    assert len(users) > 80


async def test_genshin_announcements(client: genshin.Client):
    announcements = await client.get_genshin_announcements()

    assert len(announcements) > 10


async def test_starrail_announcements(client: genshin.Client):
    announcements = await client.get_starrail_announcements()

    assert len(announcements) > 10


async def test_zzz_announcements(client: genshin.Client):
    announcements = await client.get_zzz_announcements()

    assert len(announcements) > 10


async def test_redeem_code(lclient: genshin.Client):
    try:
        await lclient.redeem_code("GENSHINGIFT")
    except genshin.InvalidCookies:
        pytest.skip("Cookie token expired.")
    except genshin.RedemptionException as e:
        pytest.skip(f"Redemption code is inconsistent: {e}")
    except genshin.AccountNotFound:
        pytest.skip("No genshin account.")


async def test_starrail_redeem_code(lclient: genshin.Client):
    try:
        await lclient.redeem_code("HSRGRANDOPEN1", game=genshin.types.Game.STARRAIL)
    except genshin.InvalidCookies:
        pytest.skip("Cookie token expired.")
    except genshin.RedemptionException as e:
        pytest.skip(f"Redemption code is inconsistent: {e}")
    except genshin.AccountNotFound:
        pytest.skip("No star rail account.")


async def test_get_accompany_characters(client: genshin.Client):
    characters = await client.get_accompany_characters()

    assert characters



import genshin


async def test_calculator_characters(client: genshin.Client):
    characters = await client.get_calculator_characters()
    assert len(characters) >= 43

    character = min(characters, key=lambda character: character.id)
    assert character.name == "Kamisato Ayaka"
    assert "enka.network" in character.icon
    assert character.max_level == 90
    assert character.level == 0
    assert not character.collab


async def test_calculator_weapons(client: genshin.Client):
    weapons = await client.get_calculator_weapons()
    assert len(weapons) >= 126

    weapon = min(weapons, key=lambda weapon: weapon.id)
    assert weapon.name == "Dull Blade"
    assert weapon.max_level == 70
    assert weapon.level == 0


async def test_calculator_artifacts(client: genshin.Client):
    artifacts = await client.get_calculator_artifacts()
    assert len(artifacts) >= 69

    artifact = min(artifacts, key=lambda artifact: artifact.id)
    assert artifact.name == "Labyrinth Wayfarer"
    assert artifact.max_level == 16
    assert artifact.level == 0


async def test_calculator_furnishings(client: genshin.Client):
    furnishings = await client.get_calculator_furnishings()
    assert len(furnishings) >= 100

    furnishing = min(furnishings, key=lambda furnishing: furnishing.id)
    assert furnishing.name == "Timber Wall With Jade Eaves"


# noqa: PT018, E277
async def test_character_talents(client: genshin.Client):
    talents = await client.get_character_talents(10000002)
    assert len(talents) == 7

    talents = sorted(talents, key=lambda t: t.group_id)

    # special character - has dash
    # fmt: off
    assert talents[0].type == "passive" and talents[0].name == "Amatsumi Kunitsumi Sanctification"
    assert talents[1].type == "passive" and talents[1].name == "Kanten Senmyou Blessing"
    assert talents[2].type == "passive" and talents[2].name == "Fruits of Shinsa"
    assert talents[3].type == "attack"  and talents[3].name == "Normal Attack: Kamisato Art - Kabuki"
    assert talents[4].type == "skill"   and talents[4].name == "Kamisato Art: Hyouka"
    assert talents[5].type == "dash"    and talents[5].name == "Kamisato Art: Senho"
    assert talents[6].type == "burst"   and talents[6].name == "Kamisato Art: Soumetsu"
    # fmt: on

    assert talents[0].max_level == 1
    assert talents[6].max_level == 10


# noqa: PT018
async def test_complete_artifact_set(client: genshin.Client):
    artifact_id = 7554  # Gladiator's Nostalgia (feather / pos #1)

    artifacts = await client.get_complete_artifact_set(artifact_id)
    artifacts = sorted(artifacts, key=lambda artifact: artifact.pos)

    assert len(artifacts) == 4
    assert artifact_id not in (a.id for a in artifacts)

    assert artifacts[0].id == 7552 and artifacts[0].name == "Gladiator's Destiny"
    assert artifacts[1].id == 7555 and artifacts[1].name == "Gladiator's Longing"
    assert artifacts[2].id == 7551 and artifacts[2].name == "Gladiator's Intoxication"
    assert artifacts[3].id == 7553 and artifacts[3].name == "Gladiator's Triumphus"


async def test_calculate(client: genshin.Client):
    cost = await (
        client.calculator()
        .set_character(10000052, current=1, target=90)
        .set_weapon(11509, current=1, target=90)
        .set_artifact_set(9651, current=0, target=20)
        .with_current_talents(current=1, target=10)
    )

    assert len(cost.character) == 11
    assert len(cost.weapon) == 12
    assert len(cost.artifacts) == 5 and all(len(i.list) == 2 for i in cost.artifacts)
    assert len(cost.talents) == 9
    assert len(cost.total) == 25
    assert cost.total[0].name == "Hero's Wit"


async def test_furnishing_calculate(client: genshin.Client):
    cost = await client.furnishings_calculator().add_furnishing(363106)

    assert cost.total[0].name == "Iron Chunk" and cost.total[0].amount == 6
    assert cost.total[1].name == "White Iron Chunk" and cost.total[1].amount == 6


async def test_calculator_characters_synced(lclient: genshin.Client):
    characters = await lclient.get_calculator_characters(sync=True)
    assert characters[0].level != 0


async def test_character_details(lclient: genshin.Client):
    # Yelan
    details = await lclient.get_character_details(10000060)
    assert details.weapon.level >= 80



import datetime

import genshin


async def test_diary(lclient: genshin.Client, genshin_uid: int):
    diary = await lclient.get_diary()
    assert diary.uid == genshin_uid == lclient.uids[genshin.Game.GENSHIN]
    assert diary.month == datetime.datetime.now().month
    assert diary.data.current_mora >= 0


async def test_diary_log(lclient: genshin.Client, genshin_uid: int):
    log = lclient.diary_log(limit=10)
    data = await log.flatten()

    if data:
        assert data[0].amount > 0

    assert log.data.uid == genshin_uid == lclient.uids[genshin.Game.GENSHIN]
    assert log.data.month == datetime.datetime.now().month



import genshin


async def test_lineup_fields(client: genshin.Client):
    data = await client.get_lineup_fields()

    assert data


async def test_lineup_scenarios(client: genshin.Client):
    data = await client.get_lineup_scenarios()

    assert data


async def test_lineups(client: genshin.Client):
    data = await client.get_lineups(limit=10, page_size=10, characters=[10000002])

    assert data


async def test_lineup_details(client: genshin.Client):
    data = await client.get_lineup_details("6358a87a30ad88f7465475e9")

    assert data


async def test_user_lineups(lclient: genshin.Client):
    await lclient.get_user_lineups()


async def test_favorite_lineups(lclient: genshin.Client):
    await lclient.get_favorite_lineups()


async def test_lineup_character_history(lclient: genshin.Client):
    data = await lclient.get_lineup_character_history()

    assert len(data) < 100



"""Modified pydantic model."""

from __future__ import annotations

import abc
import datetime
import typing
from typing import Annotated

import pydantic

from genshin.constants import CN_TIMEZONE

__all__ = ["APIModel", "Aliased", "Unique"]


class APIModel(pydantic.BaseModel):
    """Modified pydantic model."""

    model_config: pydantic.ConfigDict = pydantic.ConfigDict(arbitrary_types_allowed=True)  # type: ignore


class Unique(abc.ABC):
    """A hashable model with an id."""

    id: int

    def __int__(self) -> int:
        return hash(self.id)

    def __hash__(self) -> int:
        return hash(self.id)


def Aliased(
    alias: typing.Optional[str] = None,
    default: typing.Any = None,
    **kwargs: typing.Any,
) -> typing.Any:
    """Create an aliased field."""
    return pydantic.Field(default, alias=alias, **kwargs)


def add_timezone(value: datetime.datetime) -> datetime.datetime:
    return value.replace(tzinfo=CN_TIMEZONE)


def convert_datetime(value: typing.Optional[typing.Mapping[str, typing.Any]]) -> typing.Optional[datetime.datetime]:
    if value:
        return datetime.datetime(**value)
    return None


TZDateTime = Annotated[datetime.datetime, pydantic.AfterValidator(add_timezone)]
DateTime = Annotated[datetime.datetime, pydantic.BeforeValidator(convert_datetime)]



import pytest

import genshin


async def test_record_cards(client: genshin.Client, hoyolab_id: int):
    data = await client.get_record_cards(hoyolab_id)

    assert data

    assert data[0].level >= 40


async def test_genshin_user(client: genshin.Client, genshin_uid: int):
    data = await client.get_genshin_user(genshin_uid)

    assert data


async def test_genshin_detailed_characters(client: genshin.Client, genshin_uid: int):
    data = await client.get_genshin_detailed_characters(genshin_uid)

    assert data


async def test_partial_genshin_user(client: genshin.Client, genshin_uid: int):
    data = await client.get_partial_genshin_user(genshin_uid)

    assert data


async def test_spiral_abyss(client: genshin.Client, genshin_uid: int):
    data = await client.get_spiral_abyss(genshin_uid, previous=True)

    assert data


async def test_imaginarium_theater(client: genshin.Client, genshin_uid: int):
    data = await client.get_imaginarium_theater(genshin_uid)

    assert data


async def test_notes(lclient: genshin.Client, genshin_uid: int):
    data = await lclient.get_notes(genshin_uid)

    assert data

    if td := data.remaining_transformer_recovery_time:
        assert sum(1 for i in (td.days, td.hours, td.minutes, td.seconds) if i) == 1


async def test_genshin_activities(client: genshin.Client, genshin_uid: int):
    data = await client.get_activities(genshin_uid)

    assert data


async def test_genshin_tcg(lclient: genshin.Client, genshin_uid: int):
    data = await lclient.genshin_tcg(limit=64)

    assert data


async def test_full_genshin_user(client: genshin.Client, genshin_uid: int):
    data = await client.get_full_genshin_user(genshin_uid)

    assert data


async def test_exceptions(client: genshin.Client):
    with pytest.raises(genshin.DataNotPublic):
        await client.get_record_cards(10000000)



import genshin


async def test(client: genshin.Client):
    replicas = await client.teapot_replicas(limit=9)
    assert len(replicas) >= 9

    blueprint = replicas[0].blueprint

    furnishings = await client.get_teapot_replica_blueprint(blueprint.share_code, region=blueprint.region)
    assert len(furnishings) >= 10



"""Nox file."""

from __future__ import annotations

import logging
import pathlib
import typing

import nox

nox.options.sessions = ["reformat", "lint", "type-check", "verify-types", "test"]
nox.options.reuse_existing_virtualenvs = True
PACKAGE = "genshin"
GENERAL_TARGETS = ["./noxfile.py", "./genshin", "./tests"]
PYRIGHT_ENV = {"PYRIGHT_PYTHON_FORCE_VERSION": "latest"}
UV_RUN_EXTRA = ("uv", "run", "--isolated", "--no-dev", "--extra")
UV_RUN_NO_ISOLATE = ("uv", "run", "--no-dev", "--extra")

LOGGER = logging.getLogger("nox")


def isverbose() -> bool:
    """Whether the verbose flag is set."""
    return LOGGER.getEffectiveLevel() == logging.DEBUG - 1


def verbose_args() -> typing.Sequence[str]:
    """Return --verbose if the verbose flag is set."""
    return ["--verbose"] if isverbose() else []


@nox.session()
def docs(session: nox.Session) -> None:
    """Generate docs for this project using Pdoc."""
    output_directory = pathlib.Path("./docs/pdoc/")
    session.log("Building docs into %s", output_directory)

    session.run(*UV_RUN_NO_ISOLATE, "docs", "pdoc3", "--html", PACKAGE, "-o", str(output_directory), "--force")
    session.log("Docs generated: %s", output_directory / "index.html")


@nox.session()
def lint(session: nox.Session) -> None:
    """Run this project's modules against the pre-defined flake8 linters."""
    session.run(*UV_RUN_EXTRA, "lint", "ruff", "check", *GENERAL_TARGETS, *verbose_args())


@nox.session()
def reformat(session: nox.Session) -> None:
    """Reformat this project's modules to fit the standard style."""
    session.run(*UV_RUN_EXTRA, "reformat", "black", *GENERAL_TARGETS, *verbose_args())
    # sort __all__ and format imports
    session.run(
        *UV_RUN_EXTRA,
        "reformat",
        "ruff",
        "check",
        "--select",
        "RUF022,I",
        "--fix",
        "--preview",
        *GENERAL_TARGETS,
        *verbose_args(),
    )
    # fix all fixable linting errors
    session.run(*UV_RUN_EXTRA, "reformat", "ruff", "check", "--fix", *GENERAL_TARGETS, *verbose_args())


@nox.session(name="test")
def test(session: nox.Session) -> None:
    """Run this project's tests using pytest."""
    session.run(
        *UV_RUN_EXTRA,
        "pytest",
        "--extra",
        "all",
        "pytest",
        "--asyncio-mode=auto",
        "-r",
        "sfE",
        *verbose_args(),
        "--cov",
        PACKAGE,
        "--cov-report",
        "term",
        "--cov-report",
        "html:coverage_html",
        "--cov-report",
        "xml",
        *session.posargs,
    )


@nox.session(name="type-check")
def type_check(session: nox.Session) -> None:
    """Statically analyse and veirfy this project using pyright and mypy."""
    session.run(*UV_RUN_EXTRA, "typecheck", "--extra", "all", "pyright", PACKAGE, *verbose_args(), env=PYRIGHT_ENV)
    session.run(*UV_RUN_EXTRA, "typecheck", "--extra", "all", "mypy", PACKAGE, *verbose_args())


@nox.session(name="verify-types")
def verify_types(session: nox.Session) -> None:
    """Verify the "type completeness" of types exported by the library using pyright."""
    session.run(
        *UV_RUN_EXTRA,
        "typecheck",
        "--extra",
        "all",
        "pyright",
        "--verifytypes",
        PACKAGE,
        "--ignoreexternal",
        *verbose_args(),
        env=PYRIGHT_ENV,
    )


@nox.session(python=False)
def prettier(session: nox.Session) -> None:
    """Run prettier on markdown files."""
    session.run("prettier", "-w", "*.md", "docs/*.md", "*.yml")
